/*
 * dllcrt2.c
 *
 * Initialization code for DLLs.
 *
 * This file is part of the Mingw32 package.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  DLL support adapted from Gunther Ebert <gunther.ebert@ixos-leipzig.de>
 *
 *  Hacked by Mumit Khan <khan@xraylith.wisc.edu> to adapt to U/WIN.
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRENTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warrenties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#undef extern

#include <stdio.h>
#include <io.h>
#include <process.h>
#include <windows.h>

/*
 * Access to a standard 'main'-like argument count and list. Also included
 * is a table of environment variables.
 */
int __argc = 0;
char **__argv = 0;
char **_environ = 0;
int errno = 0;

static void
_crt_to_unix (char *arg)
{
  /* convert argv[0] to Unix specs */
  char *cp = arg;
  if (cp[1] == ':')
    {
       cp[1] = cp[0];
       cp[0] = '/';
       cp += 2;
    }
  do
    {
      if(*cp=='\\')
	*cp = '/';
    } 
  while(*cp++);
}

extern BOOL WINAPI DllMain (HANDLE, DWORD, LPVOID);

BOOL WINAPI
DllMainCRTStartup (HANDLE hDll, DWORD dwReason, LPVOID lpReserved)
{
  BOOL bRet;

  if (dwReason == DLL_PROCESS_ATTACH)
    {
      STARTUPINFO startinfo;

      GetStartupInfoA (&startinfo);
      (void) __getmainargs (&__argc, &__argv, &_environ, 0, &startinfo);

      /* convert argv[0] to be Unix like */
      _crt_to_unix (__argv[0]);

      /* From libgcc.a, calls global class constructors. */
      __main ();
    }

  /*
   * Call the user-supplied DllMain subroutine
   * NOTE: DllMain is optional, so libmingw32.a includes a stub
   *       which will be used if the user does not supply one.
   */
  bRet = DllMain (hDll, dwReason, lpReserved);

  if (dwReason == DLL_PROCESS_DETACH)
    {
      /* From libgcc.a, calls global class destructors. */
      __do_global_dtors ();
    }

  return bRet;
}
